@@ -35,7 +35,15 @@ def parse_tok(token):
         text = ''
     return bilou_tag, text
 
-def add_extra_token_pat(ambiguous_seg_pat, index_info):
+def add_extra_token_pat(ambiguous_seg_pat):
+    """extra context tokens added to ambiguous seg pat
+
+    Args:
+        ambiguous_seg_pat (str): ambiguous segmentation pattern
+
+    Returns:
+        str: ambiguous seg pat with extra context token
+    """
     extra_token_pat = r' \S+?/\S '
     ambiguous_seg_pat_with_extra_token_pat = f'{extra_token_pat}{ambiguous_seg_pat}{extra_token_pat}'
     ambiguous_seg_pat_with_extra_token_pat = ambiguous_seg_pat_with_extra_token_pat.replace('  ', ' ')
@@ -65,8 +73,8 @@ def get_ambiguous_seg_pat(tokens_in_rule, index_info):
                 ambiguous_seg_pat += f'/{bilou_tag}'
             else:
                 ambiguous_seg_pat += r'/\S'
-    if len(tokens_in_rule) < 5:
-        ambiguous_seg_pat = add_extra_token_pat(ambiguous_seg_pat, index_info)
+    if len(tokens_in_rule) < 4:
+        ambiguous_seg_pat = add_extra_token_pat(ambiguous_seg_pat)
     return ambiguous_seg_pat
 
 def construct_token_info(ambiguous_seg_candidate):
@@ -81,7 +89,12 @@ def construct_token_info(ambiguous_seg_candidate):
     token_info = ''
     for token in ambiguous_seg_candidate:
         token_parts = token.split('/')
-        token_info += f'["{token_parts[0].replace(" ", "")}"] '
+        token_text = re.search(r'(\S+)<\S+',token_parts[0]).group(1)
+        token_pos = re.search(r'<(\S+)>',token_parts[0]).group(1)
+        if token_pos != 'NO_POS':
+            token_info += f'[text="{token_text}" & pos="{token_pos}"] '
+        else:
+            token_info += f'[text="{token_text}"] '
     return token_info.strip()
 
 def get_ambiguous_seg_candidates(tokens_in_rule, index_info, bilou_tag_data):
@@ -133,8 +146,8 @@ def parse_index_info(index_info):
         index = int(index_info)
     return index
 
-def get_counter_split_suggestion(spilt_suggestion):
-    """Return opposite of split suggestion
+def get_splited_token(spilt_suggestion):
+    """Split split suggestion and return it
 
     Args:
         spilt_suggestion (str): split suggestion
@@ -142,12 +155,40 @@ def get_counter_split_suggestion(spilt_suggestion):
     Returns:
         str: opposite of split suggestion
     """
+    spilt_suggestion = spilt_suggestion.strip()
     syls = [syl.strip() for syl in spilt_suggestion.split('་') if syl and syl != ' ']
     suggestion = f'{syls[0]}་ {"་".join(syls[1:])}'
-    if spilt_suggestion[-2] == '་':
+    if spilt_suggestion[-1] == '་':
         suggestion += '་'
-    counter_split_suggestion = f' {suggestion} '
-    return counter_split_suggestion
+    splited_token = f' {suggestion} '
+    return splited_token
+
+def is_false_positive_split(tokens_in_rule, index, splited_token, human_data):
+    """Check if the rule is a false positive split case or not
+
+    Args:
+        tokens_in_rule (list): tokens in rule
+        index (int): index of token on which split is going to take
+        splited_token (str): splited token
+        human_data (str): human segmented data
+
+    Returns:
+        boolean: True if rule is false positive else false
+    """
+    split_suggestion_with_context = ''
+    splited_token = splited_token.strip()
+    for token_walker, token in enumerate(tokens_in_rule, 1):
+        token_text = re.search(r'text=\"(\S+)\"', token).group(1)
+        if token_walker == 1:
+            split_suggestion_with_context += f' {token_text} '
+        elif token_walker == index:
+            split_suggestion_with_context += f'{splited_token} '
+        else:
+            split_suggestion_with_context += f'{token_text} '
+    if split_suggestion_with_context in human_data:
+        return False
+    else:
+        return True
 
 def is_invalid_split(tokens_info, index_info, human_data):
     """Return false if split suggestion is ambiguous segmentation else true 
@@ -162,17 +203,44 @@ def is_invalid_split(tokens_info, index_info, human_data):
     """
     index = parse_index_info(index_info)
     tokens = get_tokens(tokens_info)
-    token_to_split = re.search(r'\"(\S+)\"', tokens[index-1]).group(1)
+    token_to_split = re.search(r'text=\"(\S+)\"', tokens[index-1]).group(1)
     if is_single_syl(token_to_split) or len(tokens) < index:
         return True
     else:
         split_suggestion = f" {token_to_split} "
-        counter_split_suggestion = get_counter_split_suggestion(split_suggestion)
-        if split_suggestion in human_data and counter_split_suggestion in human_data:
+        splited_token = get_splited_token(split_suggestion)
+        if split_suggestion in human_data and splited_token in human_data and not is_false_positive_split(tokens, index, splited_token, human_data):
             return False
         else:
             return True
 
+def is_false_positive_merge(tokens_in_rule, index, human_data):
+    """Check if rule is false positive merge or not
+
+    Args:
+        tokens_in_rule (list): tokens in rule
+        index (int): index of token on which merge operation is going to perform
+        human_data (str): human segmented data
+
+    Returns:
+        boolean: true if rule is false positive merge else false
+    """
+    merge_suggestion_with_context = ''
+    for token_walker, token in enumerate(tokens_in_rule, 1):
+        token_text = re.search(r'text=\"(\S+)\"', token).group(1)
+        if token_walker == 1:
+            merge_suggestion_with_context += f' {token_text} '
+        elif token_walker == index:
+            merge_suggestion_with_context += f'{token_text}'
+        elif token_walker == index+1:
+            merge_suggestion_with_context += f'{token_text} '
+        else:
+            merge_suggestion_with_context += f'{token_text} '
+    if merge_suggestion_with_context in human_data:
+        return False
+    else:
+        return True
+
 def is_invalid_merge(tokens_info, index_info, human_data):
     """Return false if merge suggestion is ambiguous segmentation else true 
 
@@ -189,11 +257,11 @@ def is_invalid_merge(tokens_info, index_info, human_data):
     if len(tokens) <= index or index == 0:
         return True
     else:
-        part1 = re.search(r'\"(\S+)\"', tokens[index-1]).group(1)
-        part2 = re.search(r'\"(\S+)\"', tokens[index]).group(1)
+        part1 = re.search(r'text=\"(\S+)\"', tokens[index-1]).group(1)
+        part2 = re.search(r'text=\"(\S+)\"', tokens[index]).group(1)
         merge_suggestion = f' {part1}{part2} '
         counter_merge_suggestion = f' {part1} {part2} '
-        if "།" not in merge_suggestion and (merge_suggestion in human_data and counter_merge_suggestion in human_data):
+        if "།" not in merge_suggestion and (merge_suggestion in human_data and counter_merge_suggestion in human_data) and not is_false_positive_merge(tokens, index, human_data):
             return False
         else:
             return True
